Index: wp-includes/default-filters.php
===================================================================
--- wp-includes/default-filters.php	(revision 15244)
+++ wp-includes/default-filters.php	(working copy)
@@ -231,8 +231,6 @@
 add_action( 'wp_scheduled_delete',        'wp_scheduled_delete'            );
 
 // Navigation menu actions
-add_action( 'trash_post',                 '_wp_trash_menu_item'            );
-add_action( 'untrash_post',               '_wp_untrash_menu_item'          );
 add_action( 'delete_post',                '_wp_delete_post_menu_item'      );
 add_action( 'delete_term',                '_wp_delete_tax_menu_item'       );
 add_action( 'transition_post_status', '_wp_menu_changing_status_observer',  10, 3 );
Index: wp-includes/nav-menu.php
===================================================================
--- wp-includes/nav-menu.php	(revision 15244)
+++ wp-includes/nav-menu.php	(working copy)
@@ -311,9 +311,6 @@
 			$original_object = get_post( $args['menu-item-object-id'] );
 			$original_parent = (int) $original_object->post_parent;
 			$original_title = $original_object->post_title;
-
-			if ( 'trash' == get_post_status( $args['menu-item-object-id'] ) )
-				return new WP_Error('update_nav_menu_item_failed', sprintf(__('The menu item "%1$s" belongs to something that is in the trash, so it cannot be updated.'), $args['menu-item-title'] ) );
 		}
 
 		if ( empty( $args['menu-item-title'] ) || $args['menu-item-title'] == $original_title ) {
@@ -655,48 +652,6 @@
 }
 
 /**
- * Callback for handling a menu item when its original object is trashed.
- *
- * @since 3.0.0
- * @access private
- *
- * @param int $object_id The ID of the original object being trashed.
- *
- */
-function _wp_trash_menu_item( $object_id = 0 ) {
-	$object_id = (int) $object_id;
-
-	$menu_item_ids = wp_get_associated_nav_menu_items( $object_id );
-
-	foreach( (array) $menu_item_ids as $menu_item_id ) {
-		$menu_item = get_post( $menu_item_id, ARRAY_A );
-		$menu_item['post_status'] = 'pending';
-		wp_insert_post($menu_item);
-	}
-}
-
-/**
- * Callback for handling a menu item when its original object is un-trashed.
- *
- * @since 3.0.0
- * @access private
- *
- * @param int $object_id The ID of the original object being untrashed.
- *
- */
-function _wp_untrash_menu_item( $object_id = 0 ) {
-	$object_id = (int) $object_id;
-
-	$menu_item_ids = wp_get_associated_nav_menu_items( $object_id );
-
-	foreach( (array) $menu_item_ids as $menu_item_id ) {
-		$menu_item = get_post( $menu_item_id, ARRAY_A );
-		$menu_item['post_status'] = 'publish';
-		wp_insert_post($menu_item);
-	}
-}
-
-/**
  * Callback for handling a menu item when its original object is deleted.
  *
  * @since 3.0.0
@@ -780,11 +735,10 @@
 
 	// give menu items draft status if their associated post objects change from "publish" to "draft", or vice versa (draft item being re-published)
 	if (
-		! empty( $post->ID ) &&
-		(
-			( 'publish' == $old_status && 'draft' == $new_status ) ||
-			( 'draft' == $old_status && 'publish' == $new_status )
-		)
+		( $new_status != $old_status ) &&
+		! empty( $post->ID ) && ! empty( $post->post_type ) &&
+		'nav_menu_item' != $post->post_type && 
+		( 'publish' == $old_status || 'publish' == $new_status )
 	) {
 		$menu_items = get_posts(array(
 			'meta_key' => '_menu_item_object_id',
@@ -796,7 +750,7 @@
 		foreach( (array) $menu_items as $menu_item ) {
 			if ( ! empty( $menu_item->ID ) ) {
 				$properties = get_object_vars( $menu_item );
-				$properties['post_status'] = $new_status;
+				$properties['post_status'] = 'publish' == $new_status ? 'publish' : 'draft';
 
 				wp_insert_post( $properties );
 			}
Index: wp-admin/includes/nav-menu.php
===================================================================
--- wp-admin/includes/nav-menu.php	(revision 15244)
+++ wp-admin/includes/nav-menu.php	(working copy)
@@ -71,13 +71,14 @@
 		$title = $item->title;
 
 		if ( isset( $item->post_status ) && 'draft' == $item->post_status ) {
-			$classes[] = 'draft';
-			/* translators: %s: title of menu item in draft status */
-			$title = sprintf( __('%s (Draft)'), $item->title );
+			$original_status = get_post_status_object( $original_object->post_status );
+			$classes[] = "draft post-status-$original_object->post_status";
+			/* translators: 1: title of menu item in draft status, 2: actual post status. */
+			$title = sprintf( __('%1$s (%2$s)'), $item->title, $original_status->label );
 		} elseif ( isset( $item->post_status ) && 'pending' == $item->post_status ) {
-			$classes[] = 'pending';
+			$classes[] = 'unsaved';
 			/* translators: %s: title of menu item in pending status */
-			$title = sprintf( __('%s (Pending)'), $item->title );
+			$title = sprintf( __('%s (Unsaved)'), $item->title );
 		}
 
 		$title = empty( $item->label ) ? $title : $item->label;
@@ -176,7 +177,27 @@
 				<div class="menu-item-actions description-wide submitbox">
 					<?php if( 'custom' != $item->type ) : ?>
 						<p class="link-to-original">
-							<?php printf( __('Original: %s'), '<a href="' . esc_attr( $item->url ) . '">' . esc_html( $original_title ) . '</a>' ); ?>
+							<?php
+							$post_status = get_post_status( $item->object_id );
+							if( 'publish' == $post_status ) {
+								printf( __('Original: %s'), '<a href="' . esc_attr( $item->url ) . '">' . esc_html( $original_title ) . '</a>', '' );
+							} else {
+								$original_url = $item->url;
+								if( 'trash' == $post_status ) {
+									$original_url = add_query_arg(
+										array(
+											'post_status' => 'trash',
+											'post_type' => $item->object,
+										),
+										admin_url( 'edit.php' )
+									);
+								}
+								$post_status_obj = get_post_status_object( $post_status );
+								/* translators: 1: title, 2: post status. */
+								printf( __('Original: %1$s (%2$s)'), '<a href="' . esc_attr( $original_url ) . '">' . esc_html( $original_title ) . '</a>',
+								$post_status_obj->label );
+							}
+							?>
 						</p>
 					<?php endif; ?>
 					<a class="item-delete submitdelete deletion" id="delete-<?php echo $item_id; ?>" href="<?php
@@ -1089,7 +1110,7 @@
 		}
 
 		if ( $some_pending_menu_items )
-			$result .= '<div class="updated inline"><p>' . __('Click Save Menu to make pending menu items public.') . '</p></div>';
+			$result .= '<div class="updated inline"><p>' . __('Click Save Menu to make unsaved menu items public.') . '</p></div>';
 
 		$result .= '<ul class="menu" id="menu-to-edit"> ';
 		$result .= walk_nav_menu_tree( array_map('wp_setup_nav_menu_item', $menu_items), 0, (object) array('walker' => $walker ) );
